core

package
v0.0.0-...-e4b14ae Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Dec 12, 2018 License: LGPL-3.0 Imports: 2 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

This section is empty.

Types

type ChainEvent

type ChainEvent struct {
	Block *types.Block
	Hash  common.Hash
}

type ChainHeadEvent

type ChainHeadEvent struct{ Block *types.Block }

type NewMinedBlockEvent

type NewMinedBlockEvent struct{ Block *types.Block }

NewMinedBlockEvent is posted when a block has been imported.

type NewTxsEvent

type NewTxsEvent struct{ Txs []*types.Transaction }

NewTxsEvent is posted when a batch of transactions enter the transaction pool.

type PendingStateEvent

type PendingStateEvent struct{}

PendingStateEvent is posted pre mining and notifies of pending state changes.

Directories

Path Synopsis

Jump to