rawdb

package
v0.0.0-...-e4b14ae Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Dec 12, 2018 License: LGPL-3.0 Imports: 9 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func DeleteBlock

func DeleteBlock(db DatabaseDeleter, hash common.Hash, number uint64)

DeleteBlock removes all block data associated with a hash.

func DeleteBody

func DeleteBody(db DatabaseDeleter, hash common.Hash, number uint64)

DeleteBody removes all block body data associated with a hash.

func DeleteCanonicalHash

func DeleteCanonicalHash(db DatabaseDeleter, number uint64)

DeleteCanonicalHash removes the number to hash canonical mapping.

func DeleteHeader

func DeleteHeader(db DatabaseDeleter, hash common.Hash, number uint64)

DeleteHeader removes all block header data associated with a hash.

func FindCommonAncestor

func FindCommonAncestor(db DatabaseReader, a, b *types.Header) *types.Header

FindCommonAncestor returns the last common ancestor of two block headers

func HasBody

func HasBody(db DatabaseReader, hash common.Hash, number uint64) bool

HasBody verifies the existence of a block body corresponding to the hash.

func HasHeader

func HasHeader(db DatabaseReader, hash common.Hash, number uint64) bool

HasHeader verifies the existence of a block header corresponding to the hash.

func ReadBlock

func ReadBlock(db DatabaseReader, hash common.Hash, number uint64) *types.Block

ReadBlock retrieves an entire block corresponding to the hash, assembling it back from the stored header and body. If either the header or body could not be retrieved nil is returned.

Note, due to concurrent download of header and block body the header and thus canonical hash can be stored in the database but the body data not (yet).

func ReadBody

func ReadBody(db DatabaseReader, hash common.Hash, number uint64) *types.Body

ReadBody retrieves the block body corresponding to the hash.

func ReadBodyRLP

func ReadBodyRLP(db DatabaseReader, hash common.Hash, number uint64) rlp.RawValue

ReadBodyRLP retrieves the block body (transactions and uncles) in RLP encoding.

func ReadCanonicalHash

func ReadCanonicalHash(db DatabaseReader, number uint64) common.Hash

ReadCanonicalHash retrieves the hash assigned to a canonical block number.

func ReadChainConfig

func ReadChainConfig(db DatabaseReader, hash common.Hash) *params.ChainConfig

ReadChainConfig retrieves the consensus settings based on the given genesis hash.

func ReadFastTrieProgress

func ReadFastTrieProgress(db DatabaseReader) uint64

ReadFastTrieProgress retrieves the number of tries nodes fast synced to allow reporting correct numbers across restarts.

func ReadHeadBlockHash

func ReadHeadBlockHash(db DatabaseReader) common.Hash

ReadHeadBlockHash retrieves the hash of the current canonical head block.

func ReadHeadFastBlockHash

func ReadHeadFastBlockHash(db DatabaseReader) common.Hash

ReadHeadFastBlockHash retrieves the hash of the current fast-sync head block.

func ReadHeadHeaderHash

func ReadHeadHeaderHash(db DatabaseReader) common.Hash

ReadHeadHeaderHash retrieves the hash of the current canonical head header.

func ReadHeader

func ReadHeader(db DatabaseReader, hash common.Hash, number uint64) *types.Header

ReadHeader retrieves the block header corresponding to the hash.

func ReadHeaderNumber

func ReadHeaderNumber(db DatabaseReader, hash common.Hash) *uint64

ReadHeaderNumber returns the header number assigned to a hash.

func ReadHeaderRLP

func ReadHeaderRLP(db DatabaseReader, hash common.Hash, number uint64) rlp.RawValue

ReadHeaderRLP retrieves a block header in its raw RLP database encoding.

func ReadTd

func ReadTd(db DatabaseReader, hash common.Hash, number uint64) *big.Int

ReadTd retrieves a block's total difficulty corresponding to the hash.

func WriteBlock

func WriteBlock(db DatabaseWriter, block *types.Block)

WriteBlock serializes a block into the database, header and body separately.

func WriteBody

func WriteBody(db DatabaseWriter, hash common.Hash, number uint64, body *types.Body)

WriteBody storea a block body into the database.

func WriteBodyRLP

func WriteBodyRLP(db DatabaseWriter, hash common.Hash, number uint64, rlp rlp.RawValue)

WriteBodyRLP stores an RLP encoded block body into the database.

func WriteCanonicalHash

func WriteCanonicalHash(db DatabaseWriter, hash common.Hash, number uint64)

WriteCanonicalHash stores the hash assigned to a canonical block number.

func WriteChainConfig

func WriteChainConfig(db DatabaseWriter, hash common.Hash, cfg *params.ChainConfig)

WriteChainConfig writes the chain config settings to the database.

func WriteHeadBlockHash

func WriteHeadBlockHash(db DatabaseWriter, hash common.Hash)

WriteHeadBlockHash stores the head block's hash.

func WriteHeadFastBlockHash

func WriteHeadFastBlockHash(db DatabaseWriter, hash common.Hash)

func WriteHeadHeaderHash

func WriteHeadHeaderHash(db DatabaseWriter, hash common.Hash)

WriteHeadHeaderHash stores the hash of the current canonical head header.

func WriteHeader

func WriteHeader(db DatabaseWriter, header *types.Header)

WriteHeader stores a block header into the database and also stores the hash- to-number mapping.

func WriteTd

func WriteTd(db DatabaseWriter, hash common.Hash, number uint64, td *big.Int)

WriteTd stores the total difficulty of a block into the database.

Types

type DatabaseDeleter

type DatabaseDeleter interface {
	Delete(key []byte) error
}

DatabaseDeleter wraps the Delete method of a backing data store.

type DatabaseReader

type DatabaseReader interface {
	Has(key []byte) (bool, error)
	Get(key []byte) ([]byte, error)
}

DatabaseReader wraps the Has and Get method of a backing data store.

type DatabaseWriter

type DatabaseWriter interface {
	Put(key []byte, value []byte) error
}

DatabaseWriter wraps the Put method of a backing data store.

Jump to