Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type ChainHeadEvent ¶
type NewMinedBlockEvent ¶
NewMinedBlockEvent is posted when a block has been imported.
type NewTxsEvent ¶
type NewTxsEvent struct{ Txs []*types.Transaction }
NewTxsEvent is posted when a batch of transactions enter the transaction pool.
type PendingStateEvent ¶
type PendingStateEvent struct{}
PendingStateEvent is posted pre mining and notifies of pending state changes.
Click to show internal directories.
Click to hide internal directories.