Documentation
¶
Index ¶
- func DeleteBlock(db DatabaseDeleter, hash common.Hash, number uint64)
- func DeleteBody(db DatabaseDeleter, hash common.Hash, number uint64)
- func DeleteCanonicalHash(db DatabaseDeleter, number uint64)
- func DeleteHeader(db DatabaseDeleter, hash common.Hash, number uint64)
- func FindCommonAncestor(db DatabaseReader, a, b *types.Header) *types.Header
- func HasBody(db DatabaseReader, hash common.Hash, number uint64) bool
- func HasHeader(db DatabaseReader, hash common.Hash, number uint64) bool
- func ReadBlock(db DatabaseReader, hash common.Hash, number uint64) *types.Block
- func ReadBody(db DatabaseReader, hash common.Hash, number uint64) *types.Body
- func ReadBodyRLP(db DatabaseReader, hash common.Hash, number uint64) rlp.RawValue
- func ReadCanonicalHash(db DatabaseReader, number uint64) common.Hash
- func ReadChainConfig(db DatabaseReader, hash common.Hash) *params.ChainConfig
- func ReadFastTrieProgress(db DatabaseReader) uint64
- func ReadHeadBlockHash(db DatabaseReader) common.Hash
- func ReadHeadFastBlockHash(db DatabaseReader) common.Hash
- func ReadHeadHeaderHash(db DatabaseReader) common.Hash
- func ReadHeader(db DatabaseReader, hash common.Hash, number uint64) *types.Header
- func ReadHeaderNumber(db DatabaseReader, hash common.Hash) *uint64
- func ReadHeaderRLP(db DatabaseReader, hash common.Hash, number uint64) rlp.RawValue
- func ReadTd(db DatabaseReader, hash common.Hash, number uint64) *big.Int
- func WriteBlock(db DatabaseWriter, block *types.Block)
- func WriteBody(db DatabaseWriter, hash common.Hash, number uint64, body *types.Body)
- func WriteBodyRLP(db DatabaseWriter, hash common.Hash, number uint64, rlp rlp.RawValue)
- func WriteCanonicalHash(db DatabaseWriter, hash common.Hash, number uint64)
- func WriteChainConfig(db DatabaseWriter, hash common.Hash, cfg *params.ChainConfig)
- func WriteHeadBlockHash(db DatabaseWriter, hash common.Hash)
- func WriteHeadFastBlockHash(db DatabaseWriter, hash common.Hash)
- func WriteHeadHeaderHash(db DatabaseWriter, hash common.Hash)
- func WriteHeader(db DatabaseWriter, header *types.Header)
- func WriteTd(db DatabaseWriter, hash common.Hash, number uint64, td *big.Int)
- type DatabaseDeleter
- type DatabaseReader
- type DatabaseWriter
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func DeleteBlock ¶
func DeleteBlock(db DatabaseDeleter, hash common.Hash, number uint64)
DeleteBlock removes all block data associated with a hash.
func DeleteBody ¶
func DeleteBody(db DatabaseDeleter, hash common.Hash, number uint64)
DeleteBody removes all block body data associated with a hash.
func DeleteCanonicalHash ¶
func DeleteCanonicalHash(db DatabaseDeleter, number uint64)
DeleteCanonicalHash removes the number to hash canonical mapping.
func DeleteHeader ¶
func DeleteHeader(db DatabaseDeleter, hash common.Hash, number uint64)
DeleteHeader removes all block header data associated with a hash.
func FindCommonAncestor ¶
func FindCommonAncestor(db DatabaseReader, a, b *types.Header) *types.Header
FindCommonAncestor returns the last common ancestor of two block headers
func HasBody ¶
func HasBody(db DatabaseReader, hash common.Hash, number uint64) bool
HasBody verifies the existence of a block body corresponding to the hash.
func HasHeader ¶
func HasHeader(db DatabaseReader, hash common.Hash, number uint64) bool
HasHeader verifies the existence of a block header corresponding to the hash.
func ReadBlock ¶
ReadBlock retrieves an entire block corresponding to the hash, assembling it back from the stored header and body. If either the header or body could not be retrieved nil is returned.
Note, due to concurrent download of header and block body the header and thus canonical hash can be stored in the database but the body data not (yet).
func ReadBodyRLP ¶
ReadBodyRLP retrieves the block body (transactions and uncles) in RLP encoding.
func ReadCanonicalHash ¶
func ReadCanonicalHash(db DatabaseReader, number uint64) common.Hash
ReadCanonicalHash retrieves the hash assigned to a canonical block number.
func ReadChainConfig ¶
func ReadChainConfig(db DatabaseReader, hash common.Hash) *params.ChainConfig
ReadChainConfig retrieves the consensus settings based on the given genesis hash.
func ReadFastTrieProgress ¶
func ReadFastTrieProgress(db DatabaseReader) uint64
ReadFastTrieProgress retrieves the number of tries nodes fast synced to allow reporting correct numbers across restarts.
func ReadHeadBlockHash ¶
func ReadHeadBlockHash(db DatabaseReader) common.Hash
ReadHeadBlockHash retrieves the hash of the current canonical head block.
func ReadHeadFastBlockHash ¶
func ReadHeadFastBlockHash(db DatabaseReader) common.Hash
ReadHeadFastBlockHash retrieves the hash of the current fast-sync head block.
func ReadHeadHeaderHash ¶
func ReadHeadHeaderHash(db DatabaseReader) common.Hash
ReadHeadHeaderHash retrieves the hash of the current canonical head header.
func ReadHeader ¶
ReadHeader retrieves the block header corresponding to the hash.
func ReadHeaderNumber ¶
func ReadHeaderNumber(db DatabaseReader, hash common.Hash) *uint64
ReadHeaderNumber returns the header number assigned to a hash.
func ReadHeaderRLP ¶
ReadHeaderRLP retrieves a block header in its raw RLP database encoding.
func WriteBlock ¶
func WriteBlock(db DatabaseWriter, block *types.Block)
WriteBlock serializes a block into the database, header and body separately.
func WriteBodyRLP ¶
WriteBodyRLP stores an RLP encoded block body into the database.
func WriteCanonicalHash ¶
func WriteCanonicalHash(db DatabaseWriter, hash common.Hash, number uint64)
WriteCanonicalHash stores the hash assigned to a canonical block number.
func WriteChainConfig ¶
func WriteChainConfig(db DatabaseWriter, hash common.Hash, cfg *params.ChainConfig)
WriteChainConfig writes the chain config settings to the database.
func WriteHeadBlockHash ¶
func WriteHeadBlockHash(db DatabaseWriter, hash common.Hash)
WriteHeadBlockHash stores the head block's hash.
func WriteHeadFastBlockHash ¶
func WriteHeadFastBlockHash(db DatabaseWriter, hash common.Hash)
func WriteHeadHeaderHash ¶
func WriteHeadHeaderHash(db DatabaseWriter, hash common.Hash)
WriteHeadHeaderHash stores the hash of the current canonical head header.
func WriteHeader ¶
func WriteHeader(db DatabaseWriter, header *types.Header)
WriteHeader stores a block header into the database and also stores the hash- to-number mapping.
Types ¶
type DatabaseDeleter ¶
DatabaseDeleter wraps the Delete method of a backing data store.
type DatabaseReader ¶
DatabaseReader wraps the Has and Get method of a backing data store.
type DatabaseWriter ¶
DatabaseWriter wraps the Put method of a backing data store.